<?php

namespace Drupal\fsfpay\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Entity\PaymentMethodInterface;
use Drupal\commerce_payment\Exception\HardDeclineException;
use Drupal\commerce_payment\Exception\InvalidRequestException;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OnsitePaymentGatewayBase;
use Drupal\commerce_price\Price;
use Drupal\Core\Form\FormStateInterface;
use GuzzleHttp\Exception\RequestException;
use InvalidArgumentException;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\fsfpay\Plugin\Commerce\PaymentGateway\FsfpayInterface;

/**
 * Provides the PayPal Payflow payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "fsfpay",
 *   label = "Fsfpay",
 *   display_label = @Translation("FSFPay"),
 *    forms = {
 *     "add-payment-method" = "Drupal\fsfpay\PluginForm\PaymentMethodAddForm",
 *   },
 *   payment_method_types = {"fsfpay"},
 * )
 */
class Fsfpay extends OnsitePaymentGatewayBase implements FsfpayInterface
{

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * The rounder.
   *
   * @var \Drupal\commerce_price\RounderInterface
   */
  protected $rounder;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition)
  {
    $instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $instance->httpClient = $container->get('http_client');
    $instance->rounder = $container->get('commerce_price.rounder');
    return $instance;
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration()
  {
    return [
      'api_username' => '',
      'api_key' => '',
      'secret_key' => '',
    ] + parent::defaultConfiguration();
  }

  /**
   * {@inheritdoc}
   */
  public function buildConfigurationForm(array $form, FormStateInterface $form_state)
  {
    $form = parent::buildConfigurationForm($form, $form_state);

    $form['api_username'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Api Username'),
      '#default_value' => $this->configuration['api_username'],
      '#required' => TRUE,
    ];
    $form['api_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Api Key'),
      '#default_value' => $this->configuration['api_key'],
      '#required' => TRUE,
    ];
    $form['secret_key'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Secret Key'),
      '#default_value' => $this->configuration['secret_key'],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
  {
    parent::submitConfigurationForm($form, $form_state);

    if (!$form_state->getErrors()) {
      $values = $form_state->getValue($form['#parents']);

      $this->configuration['api_username'] = $values['api_username'];
      $this->configuration['api_key'] = $values['api_key'];
      $this->configuration['secret_key'] = $values['secret_key'];
    }
  }

  /**
   * Returns the api username.
   */
  protected function getApiUsername()
  {
    return $this->configuration['api_username'] ?: '';
  }

  /**
   * Returns the api key.
   */
  protected function getApiKey()
  {
    return $this->configuration['api_key'] ?: '';
  }

  /**
   * Returns the secret key.
   */
  protected function getSecretKey()
  {
    return $this->configuration['secret_key'] ?: '';
  }

  /**
   * Merge default Payflow parameters in with the provided ones.
   *
   * @param array $parameters
   *   The parameters for the transaction.
   *
   * @return array
   *   The new parameters.
   */
  protected function getParameters(array $parameters = [])
  {
    $defaultParameters = [
      'api_username' => $this->getApiUsername(),
      'api_key' => $this->getApiKey(),
      'secret_key' => $this->getSecretKey(),
    ];

    return $parameters + $defaultParameters;
  }

  /**
   * Attempt to validate payment information according to a payment state.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentInterface $payment
   *   The payment to validate.
   * @param string|null $payment_state
   *   The payment state to validate the payment for.
   */
  protected function validatePayment(PaymentInterface $payment, $payment_state = 'new')
  {
    $this->assertPaymentState($payment, [$payment_state]);

    $payment_method = $payment->getPaymentMethod();
    if (empty($payment_method)) {
      throw new InvalidArgumentException('The provided payment has no payment method referenced.');
    }

    switch ($payment_state) {
      case 'new':
        if ($payment_method->isExpired()) {
          throw new HardDeclineException('The provided payment method has expired.');
        }

        break;

      case 'authorization':
        if ($payment->isExpired()) {
          throw new \InvalidArgumentException('Authorizations are guaranteed for up to 29 days.');
        }
        if (empty($payment->getRemoteId())) {
          throw new \InvalidArgumentException('Could not retrieve the transaction ID.');
        }
        break;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function createPayment(PaymentInterface $payment, $capture = TRUE)
  {

    $this->validatePayment($payment, 'new');

    echo "<pre>";
    $this->assertPaymentState($payment, ['new']);
    $payment_method = $payment->getPaymentMethod();
    // print_r($payment_method);
    $this->assertPaymentMethod($payment_method);

    $paid_amount = $payment->getAmount();
    $currency = $paid_amount->getCurrencyCode();

    $billing = $payment_method->getBillingProfile();
    /** @var \Drupal\address\Plugin\Field\FieldType\AddressItem $address */
    $address = $billing->get('address')->first();

    $mode = $this->getMode();
    $url = 'checkout/' . $payment->getOrderId() . '/payment';
    $callbackUrl = \Drupal\Core\Url::fromUserInput('/' . $url,  array('absolute' => 'true'))->toString();
    $url = 'checkout/' . $payment->getOrderId() . '/complete';
    $successRedirectUrl = \Drupal\Core\Url::fromUserInput('/' . $url,  array('absolute' => 'true'))->toString();

    if ($mode == 'test') {
      $test_mode = 1;
    } else {
      $test_mode = 0;
    }
    echo $callbackUrl;

    // Load the current user.
    $current_user = \Drupal::currentUser();
    $user_mail = $current_user->getEmail();

    /* You should fetch order details from database. */
    $data = [
      'apiKey' => $this->getApiKey(),
      'apiUsername' => $this->getApiUsername(),
      'orderId' => $payment->getOrderId(),
      'email' => $user_mail,
      'amount' => $paid_amount->getNumber(),
      'currency' => $currency,
      'cryptocurrency' => 'BTC',
      'testmode' => $test_mode,
      'callbackUrl' => 'https://box4u.co.il',
      'successRedirectUrl' => '',
      'language' => 'en'
    ];
    $response = $this->getPaymentWindow($data);

    if ($response['success']) {
      echo '<script src="' . $response['scriptUrl'] . '"></script>';
      dd($user_mail);
    } else {
      echo 'Payment Error: ' . $response['message'];
      die();
    }

    $remote_id = $payment_method->getRemoteId();
    $next_state = $capture ? 'completed' : 'authorization';
    $payment->setState($next_state);
    $payment->$response['paymentId'];
    $payment->setExpiresTime(strtotime('+5 days'));
    $payment->save();
  }

  /**
   * {@inheritdoc}
   */
  public function capturePayment(PaymentInterface $payment, Price $amount = NULL)
  {
  }

  /**
   * {@inheritdoc}
   */
  public function voidPayment(PaymentInterface $payment)
  {
  }

  /**
   * {@inheritdoc}
   *
   * TODO: Find a way to store the capture ID.
   */
  public function refundPayment(PaymentInterface $payment, Price $amount = NULL)
  {
  }

  /**
   * {@inheritdoc}
   */
  public function createPaymentMethod(PaymentMethodInterface $payment_method, array $payment_details)
  {

    $remote_id = $payment_details['payment_method_nonce'];
    $payment_method->setRemoteId($remote_id);
    $expires = 2; //$this->time->getRequestTime() + (3600 * 24) - 5;
    $payment_method->setExpiresTime($expires);
    $payment_method->save();
  }

  /**
   * {@inheritdoc}
   */
  public function deletePaymentMethod(PaymentMethodInterface $payment_method)
  {
    $payment_method->delete();
  }



  function getPaymentWindow($reqData = [])
  {
    $ch = curl_init();
    $data['requestHash'] = hash('sha256', $this->getApiKey() . $this->getApiUsername() . $this->getSecretKey());
    $data = array_merge($data, $reqData);
    try {
      curl_setopt_array($ch, [
        CURLOPT_URL => 'https://panel.fsfpay.com/api/payment',
        CURLOPT_HTTPHEADER => [
          'Content-Type: application/x-www-form-urlencoded'
        ],
        CURLOPT_POST => true,
        CURLOPT_RETURNTRANSFER => true,
        CURLOPT_POSTFIELDS => http_build_query($data)
      ]);
      $response = curl_exec($ch);
      $response = json_decode($response, true);
      echo "<pre>";
      // dd($payment);
      return $response;
    } catch (RequestException $e) {
      throw new HardDeclineException('Could not charge the payment method.');
    }
  }
}
